@@ -19,43 +19,49 @@
  *  Authors:
  *    Lauris Kaplinski <lauris@ximian.com>
  *
- *  Copyright (C) 2000-2001 Ximian Inc. and authors
- *
+ *  Copyright (C) 2000-2003 Ximian Inc. and authors
  */
 
-#define __GNOME_GLYPHLIST_C__
-
-#include "config.h"
+#include <config.h>
 #include <string.h>
-#include "gnome-glyphlist-private.h"
 
-static void ggl_ensure_glyph_space (GnomeGlyphList * gl, gint space);
-static void ggl_ensure_rule_space (GnomeGlyphList * gl, gint space);
+#include <libgnomeprint/gnome-glyphlist-private.h>
+
+static void ggl_ensure_glyph_space (GnomeGlyphList *gl, gint space);
+static void ggl_ensure_rule_space (GnomeGlyphList *gl, gint space);
 
 #define GGL_ENSURE_GLYPH_SPACE(ggl,s) {if ((ggl)->g_size < (ggl)->g_length + (s)) ggl_ensure_glyph_space (ggl, s);}
 #define GGL_ENSURE_RULE_SPACE(ggl,s) {if((ggl)->r_size < (ggl)->r_length + (s)) ggl_ensure_rule_space (ggl, s);}
 
-void gnome_glyphlist_moveto_x (GnomeGlyphList * gl, gdouble distance);
-void gnome_glyphlist_moveto_y (GnomeGlyphList * gl, gdouble distance);
-void gnome_glyphlist_rmoveto_x (GnomeGlyphList * gl, gdouble distance);
-void gnome_glyphlist_rmoveto_y (GnomeGlyphList * gl, gdouble distance);
-void gnome_glyphlist_push_cp (GnomeGlyphList * gl);
-void gnome_glyphlist_pop_cp (GnomeGlyphList * gl);
+void gnome_glyphlist_moveto_x  (GnomeGlyphList *gl, gdouble distance);
+void gnome_glyphlist_moveto_y  (GnomeGlyphList *gl, gdouble distance);
+void gnome_glyphlist_rmoveto_x (GnomeGlyphList *gl, gdouble distance);
+void gnome_glyphlist_rmoveto_y (GnomeGlyphList *gl, gdouble distance);
+
+
+GType
+gnome_glyphlist_get_type(void)
+{
+	static GType type = 0;
+	
+	if (type == 0) {
+		type = g_boxed_type_register_static
+			("GnomeGlyphList",
+			 (GBoxedCopyFunc) gnome_glyphlist_ref,
+			 (GBoxedFreeFunc) gnome_glyphlist_unref);
+	}
+
+	return type;
+}
 
 GnomeGlyphList *
 gnome_glyphlist_new (void)
 {
 	GnomeGlyphList *gl;
 
-	gl = g_new (GnomeGlyphList, 1);
+	gl = g_new0 (GnomeGlyphList, 1);
 
 	gl->refcount = 1;
-	gl->glyphs = NULL;
-	gl->g_length = 0;
-	gl->g_size = 0;
-	gl->rules = NULL;
-	gl->r_length = 0;
-	gl->r_size = 0;
 
 	return gl;
 }
@@ -109,17 +115,20 @@ gnome_glyphlist_duplicate (GnomeGlyphList *gl)
 	new = g_new (GnomeGlyphList, 1);
 
 	new->refcount = 1;
-	new->glyphs = g_new (gint, gl->g_length);
-	memcpy (new->glyphs, gl->glyphs, gl->g_length * sizeof (gint));
-	new->g_length = gl->g_length;
-	new->g_size = gl->g_length;
-	new->rules = g_new (GGLRule, gl->r_length);
-	memcpy (new->rules, gl->rules, gl->r_length * sizeof (GGLRule));
-	new->r_length = gl->r_length;
-	new->r_size = gl->r_length;
+	new->glyphs    = g_new (gint,    gl->g_length);
+	new->rules     = g_new (GGLRule, gl->r_length);
 
+	new->g_length = gl->g_length;
+	new->g_size   = gl->g_length;
+	new->r_length = gl->r_length;
+	new->r_size   = gl->r_length;
+
+	memcpy (new->glyphs, gl->glyphs, gl->g_length * sizeof (gint));
+	memcpy (new->rules,  gl->rules,  gl->r_length * sizeof (GGLRule));
+	
 	for (i = 0; i < new->r_length; i++) {
-		if (new->rules[i].code == GGL_FONT) gnome_font_ref (new->rules[i].value.font);
+		if (new->rules[i].code == GGL_FONT)
+			gnome_font_ref (new->rules[i].value.font);
 	}
 
 	return new;
@@ -128,13 +137,23 @@ gnome_glyphlist_duplicate (GnomeGlyphList *gl)
 gboolean
 gnome_glyphlist_check (const GnomeGlyphList *gl, gboolean rules)
 {
-	if (gl == NULL) return FALSE;
+	if (gl == NULL)
+		return FALSE;
 
 	return TRUE;
 }
 
+/**
+ * gnome_glyphlist_glyph:
+ * @gl: 
+ * @glyph: 
+ * 
+ * Appends a single glyph to the glyphlist. It will be connected
+ * to the previous glyphs by the previously defined rules
+ *
+ **/
 void
-gnome_glyphlist_glyph (GnomeGlyphList * gl, gint glyph)
+gnome_glyphlist_glyph (GnomeGlyphList *gl, gint glyph)
 {
 	g_return_if_fail (gl != NULL);
 	g_return_if_fail (GNOME_IS_GLYPHLIST (gl));
@@ -146,8 +165,33 @@ gnome_glyphlist_glyph (GnomeGlyphList * gl, gint glyph)
 	gl->g_length++;
 }
 
+/**
+ * gnome_glyphlist_glyphs:
+ * @gl: 
+ * @glyphs: 
+ * @num_glyphs: 
+ * 
+ * Append a string of glyphs
+ **/
 void
-gnome_glyphlist_moveto_x (GnomeGlyphList * gl, gdouble distance)
+gnome_glyphlist_glyphs (GnomeGlyphList *gl, gint *glyphs, gint num_glyphs)
+{
+	gint i;
+
+	g_return_if_fail (gl != NULL);
+	g_return_if_fail (GNOME_IS_GLYPHLIST (gl));
+	g_return_if_fail (glyphs != NULL);
+
+	GGL_ENSURE_GLYPH_SPACE (gl, num_glyphs);
+
+	for (i = 0; i < num_glyphs; i++) {
+		gnome_glyphlist_glyph (gl, glyphs[i]);
+	}
+}
+
+
+void
+gnome_glyphlist_moveto_x (GnomeGlyphList *gl, gdouble distance)
 {
 	gint r;
 
@@ -187,7 +231,7 @@ gnome_glyphlist_moveto_x (GnomeGlyphList * gl, gdouble distance)
 }
 
 void
-gnome_glyphlist_rmoveto_x (GnomeGlyphList * gl, gdouble distance)
+gnome_glyphlist_rmoveto_x (GnomeGlyphList *gl, gdouble distance)
 {
 	gint r;
 
@@ -226,7 +270,7 @@ gnome_glyphlist_rmoveto_x (GnomeGlyphList * gl, gdouble distance)
 }
 
 void
-gnome_glyphlist_moveto_y (GnomeGlyphList * gl, gdouble distance)
+gnome_glyphlist_moveto_y (GnomeGlyphList *gl, gdouble distance)
 {
 	gint r;
 
@@ -266,7 +310,7 @@ gnome_glyphlist_moveto_y (GnomeGlyphList * gl, gdouble distance)
 }
 
 void
-gnome_glyphlist_rmoveto_y (GnomeGlyphList * gl, gdouble distance)
+gnome_glyphlist_rmoveto_y (GnomeGlyphList *gl, gdouble distance)
 {
 	gint r;
 
@@ -304,8 +348,16 @@ gnome_glyphlist_rmoveto_y (GnomeGlyphList * gl, gdouble distance)
 	gl->r_length++;
 }
 
+/**
+ * gnome_glyphlist_moveto:
+ * @gl: 
+ * @x: 
+ * @y: 
+ * 
+ * Position manually the glyph following
+ **/
 void
-gnome_glyphlist_moveto (GnomeGlyphList * gl, gdouble x, gdouble y)
+gnome_glyphlist_moveto (GnomeGlyphList *gl, gdouble x, gdouble y)
 {
 	g_return_if_fail (gl != NULL);
 	g_return_if_fail (GNOME_IS_GLYPHLIST (gl));
@@ -314,8 +366,17 @@ gnome_glyphlist_moveto (GnomeGlyphList * gl, gdouble x, gdouble y)
 	gnome_glyphlist_moveto_y (gl, y);
 }
 
+/**
+ * gnome_glyphlist_rmoveto:
+ * @gl: 
+ * @x: 
+ * @y: 
+ * 
+ * Position the glyph following relative to current pen position
+ *
+ **/
 void
-gnome_glyphlist_rmoveto (GnomeGlyphList * gl, gdouble x, gdouble y)
+gnome_glyphlist_rmoveto (GnomeGlyphList *gl, gdouble x, gdouble y)
 {
 	g_return_if_fail (gl != NULL);
 	g_return_if_fail (GNOME_IS_GLYPHLIST (gl));
@@ -324,68 +385,17 @@ gnome_glyphlist_rmoveto (GnomeGlyphList * gl, gdouble x, gdouble y)
 	gnome_glyphlist_rmoveto_y (gl, y);
 }
 
+/**
+ * gnome_glyphlist_advance:
+ * @gl: 
+ * @advance: 
+ *
+ * Whether or not to move the pen position by the font standard
+ * advance vector. Advancing happens immediately after glyph is
+ * sent through pipeline.
+ **/
 void
-gnome_glyphlist_push_cp (GnomeGlyphList * gl)
-{
-	gint r;
-
-	g_return_if_fail (gl != NULL);
-	g_return_if_fail (GNOME_IS_GLYPHLIST (gl));
-
-	for (r = gl->r_length - 1; r >= 0; r--) {
-		if (gl->rules[r].code == GGL_POSITION) {
-			g_return_if_fail (gl->rules[r].value.ival <= gl->g_length);
-			if (gl->rules[r].value.ival == gl->g_length) {
-				/* There is ruleset at the end of glyphlist */
-				GGL_ENSURE_RULE_SPACE (gl, 1);
-				gl->rules[r].code = GGL_PUSHCP;
-				gl->r_length++;
-				return;
-			}
-			break;
-		}
-	}
-
-	GGL_ENSURE_RULE_SPACE (gl, 2);
-	gl->rules[gl->r_length].code = GGL_POSITION;
-	gl->rules[gl->r_length].value.ival = gl->g_length;
-	gl->r_length++;
-	gl->rules[gl->r_length].code = GGL_PUSHCP;
-	gl->r_length++;
-}
-
-void
-gnome_glyphlist_pop_cp (GnomeGlyphList * gl)
-{
-	gint r;
-
-	g_return_if_fail (gl != NULL);
-	g_return_if_fail (GNOME_IS_GLYPHLIST (gl));
-
-	for (r = gl->r_length - 1; r >= 0; r--) {
-		if (gl->rules[r].code == GGL_POSITION) {
-			g_return_if_fail (gl->rules[r].value.ival <= gl->g_length);
-			if (gl->rules[r].value.ival == gl->g_length) {
-				/* There is ruleset at the end of glyphlist */
-				GGL_ENSURE_RULE_SPACE (gl, 1);
-				gl->rules[r].code = GGL_POPCP;
-				gl->r_length++;
-				return;
-			}
-			break;
-		}
-	}
-
-	GGL_ENSURE_RULE_SPACE (gl, 2);
-	gl->rules[gl->r_length].code = GGL_POSITION;
-	gl->rules[gl->r_length].value.ival = gl->g_length;
-	gl->r_length++;
-	gl->rules[gl->r_length].code = GGL_POPCP;
-	gl->r_length++;
-}
-
-void
-gnome_glyphlist_advance (GnomeGlyphList * gl, gboolean advance)
+gnome_glyphlist_advance (GnomeGlyphList *gl, gboolean advance)
 {
 	gint r;
 
@@ -422,8 +432,21 @@ gnome_glyphlist_advance (GnomeGlyphList * gl, gboolean advance)
 	gl->r_length++;
 }
 
+/**
+ * gnome_glyphlist_letterspace:
+ * @gl: 
+ * @letterspace:
+ *
+ * Amount of white space to add between glyphs connected by
+ * an advance rule. It is specified in font units (i.e. 12 for 12pt
+ * font is the width of an em square). If glyph is manually positioned,
+ * letterspace value will be ignored. Letterspace will be added
+ * immediately before placing a new glyph
+ * FIXME: what does the last sentence mean? (Chema)
+ *
+ **/
 void
-gnome_glyphlist_letterspace (GnomeGlyphList * gl, gdouble letterspace)
+gnome_glyphlist_letterspace (GnomeGlyphList *gl, gdouble letterspace)
 {
 	gint r;
 
@@ -460,8 +483,21 @@ gnome_glyphlist_letterspace (GnomeGlyphList * gl, gdouble letterspace)
 	gl->r_length++;
 }
 
+
+
+/**
+ * gnome_glyphlist_kerning:
+ * @gl: 
+ * @kerning:
+ *
+ * Amount of kerning to add between glyphs connected by an advance
+ * rule. It is specified as fraction of a full kerning value 
+ * If a glyph is manually positioned, the kerning value is ignored
+ * Kerning will be added immediately before placing a new glyph
+ * FIXME: What does the last sentence mean? (Chema)
+ **/
 void
-gnome_glyphlist_kerning (GnomeGlyphList * gl, gdouble kerning)
+gnome_glyphlist_kerning (GnomeGlyphList *gl, gdouble kerning)
 {
 	gint r;
 
@@ -498,8 +534,15 @@ gnome_glyphlist_kerning (GnomeGlyphList * gl, gdouble kerning)
 	gl->r_length++;
 }
 
+/**
+ * gnome_glyphlist_font:
+ * @gl: 
+ * @font: 
+ * 
+ * Specify the font to be used for the glyphs that follow
+ **/
 void
-gnome_glyphlist_font (GnomeGlyphList * gl, GnomeFont * font)
+gnome_glyphlist_font (GnomeGlyphList *gl, GnomeFont * font)
 {
 	gint r;
 
@@ -542,8 +585,17 @@ gnome_glyphlist_font (GnomeGlyphList * gl, GnomeFont * font)
 	gl->r_length++;
 }
 
+/**
+ * gnome_glyphlist_color:
+ * @gl: 
+ * @color: 
+ * 
+ * Specify the color as RRGGBBAA to be used for the glyphs
+ * that follow
+ *
+ **/
 void
-gnome_glyphlist_color (GnomeGlyphList * gl, guint32 color)
+gnome_glyphlist_color (GnomeGlyphList *gl, guint32 color)
 {
 	gint r;
 
@@ -580,12 +632,28 @@ gnome_glyphlist_color (GnomeGlyphList * gl, guint32 color)
 	gl->r_length++;
 }
 
+/**
+ * gnome_glyphlist_from_text_sized_dumb:
+ * @font: 
+ * @color: 
+ * @kerning: 
+ * @letterspace: 
+ * @text: 
+ * @length: 
+ * 
+ * Appends utf8 text, converting it to glyphs and connecting it
+ * as specified by rules. You cannot expect anything about
+ * language-specific typesetting rules, so if the given script
+ * does not use trivial placement, you should better avoid this
+ * 
+ * Return Value: 
+ **/
 GnomeGlyphList *
 gnome_glyphlist_from_text_sized_dumb (GnomeFont * font, guint32 color,
 				      gdouble kerning, gdouble letterspace,
 				      const guchar * text, gint length)
 {
-	GnomeGlyphList * gl;
+	GnomeGlyphList *gl;
 	const guchar * p;
 
 	g_return_val_if_fail (font != NULL, NULL);
@@ -609,6 +677,18 @@ gnome_glyphlist_from_text_sized_dumb (GnomeFont * font, guint32 color,
 	return gl;
 }
 
+/**
+ * gnome_glyphlist_from_text_dumb:
+ * @font: 
+ * @color: 
+ * @kerning: 
+ * @letterspace: 
+ * @text: 
+ * 
+ * See _from_text_sized_dumb
+ * 
+ * Return Value: 
+ **/
 GnomeGlyphList *
 gnome_glyphlist_from_text_dumb (GnomeFont * font, guint32 color,
 				gdouble kerning, gdouble letterspace,
@@ -623,8 +703,19 @@ gnome_glyphlist_from_text_dumb (GnomeFont * font, guint32 color,
 						     text, strlen (text));
 }
 
+/**
+ * gnome_glyphlist_text_sized_dumb:
+ * @gl: 
+ * @text: 
+ * @length: 
+ * 
+ * The 'dumb' versions of glyphlist creation
+ * It just places glyphs one after another - no ligaturing etc.
+ * text is utf8, of course
+ *
+ **/
 void
-gnome_glyphlist_text_sized_dumb (GnomeGlyphList * gl, const guchar * text, gint length)
+gnome_glyphlist_text_sized_dumb (GnomeGlyphList *gl, const guchar *text, gint length)
 {
 	GnomeFont * font;
 	const guchar * p;
@@ -634,7 +725,8 @@ gnome_glyphlist_text_sized_dumb (GnomeGlyphList * gl, const guchar * text, gint
 	g_return_if_fail (GNOME_IS_GLYPHLIST (gl));
 	g_return_if_fail (text != NULL);
 
-	if (length < 1) return;
+	if (length < 1)
+		return;
 
 	font = NULL;
 	for (r = gl->r_length - 1; r >= 0; r--) {
@@ -653,8 +745,15 @@ gnome_glyphlist_text_sized_dumb (GnomeGlyphList * gl, const guchar * text, gint
 	}
 }
 
+/**
+ * gnome_glyphlist_text_dumb:
+ * @gl: 
+ * @text: 
+ * 
+ * See _text_sized_dumb
+ **/
 void
-gnome_glyphlist_text_dumb (GnomeGlyphList * gl, const guchar * text)
+gnome_glyphlist_text_dumb (GnomeGlyphList *gl, const guchar *text)
 {
 	g_return_if_fail (gl != NULL);
 	g_return_if_fail (GNOME_IS_GLYPHLIST (gl));
@@ -663,36 +762,24 @@ gnome_glyphlist_text_dumb (GnomeGlyphList * gl, const guchar * text)
 	gnome_glyphlist_text_sized_dumb (gl, text, strlen (text));
 }
 
-void
-gnome_glyphlist_glyphs (GnomeGlyphList * gl, gint * glyphs, gint num_glyphs)
-{
-	gint i;
-
-	g_return_if_fail (gl != NULL);
-	g_return_if_fail (GNOME_IS_GLYPHLIST (gl));
-	g_return_if_fail (glyphs != NULL);
-
-	GGL_ENSURE_GLYPH_SPACE (gl, num_glyphs);
-
-	for (i = 0; i < num_glyphs; i++) {
-		gnome_glyphlist_glyph (gl, glyphs[i]);
-	}
-}
-
 static void
-ggl_ensure_glyph_space (GnomeGlyphList * gl, gint space)
+ggl_ensure_glyph_space (GnomeGlyphList *gl, gint space)
 {
-	if (gl->g_size >= gl->g_length + space) return;
+	if (gl->g_size >= gl->g_length + space)
+		return;
 
+	/* FIXME: This is wrong! what if space is greater than BLOCK_SIZE? (Chema) */
 	gl->g_size += GGL_RULE_BLOCK_SIZE;
 	gl->glyphs = g_renew (gint, gl->glyphs, gl->g_size);
 }
 
 static void
-ggl_ensure_rule_space (GnomeGlyphList * gl, gint space)
+ggl_ensure_rule_space (GnomeGlyphList *gl, gint space)
 {
-	if (gl->r_size >= gl->r_length + space) return;
+	if (gl->r_size >= gl->r_length + space)
+		return;
 
+	/* FIXME: This is wrong! what if space is greater than BLOCK_SIZE? (Chema) */
 	gl->r_size += GGL_RULE_BLOCK_SIZE;
 	gl->rules = g_renew (GGLRule, gl->rules, gl->r_size);
 }
