@@ -23,12 +23,16 @@
  *  Copyright 2001-2003 Ximian, Inc.
  */
 
+#define GNOME_PRINT_UNSTABLE_API
+
 #include <config.h>
 #include <ctype.h>
 #include <stdlib.h>
 #include <locale.h>
+#include <string.h>
 
 #include <libgnomeprint/gpa/gpa-config.h>
+#include <libgnomeprint/gpa/gpa-key.h>
 #include <libgnomeprint/gnome-print-config.h>
 #include <libgnomeprint/gnome-print-config-private.h>
 #include <libgnomeprint/gnome-print-job.h>
@@ -155,16 +159,53 @@ gnome_print_config_dup (GnomePrintConfig *old_config)
 	return config;
 }
 
+/**
+ * gnome_print_config_keys_compat:
+ * @key: 
+ * 
+ * Used for backward compat when keys change. Known to need love.
+ * 
+ * Return Value: 
+ **/
+static gchar *
+gnome_print_config_keys_compat (const gchar *key)
+{
+	const gchar *old [] = {
+		"Settings.Transport.Backend.FileName",
+		NULL
+	};
+	const gchar *new [] = {
+		GNOME_PRINT_KEY_OUTPUT_FILENAME,
+		NULL
+	};
+	gint i = 0;
+
+	while (old[i]) {
+		if (strcmp (old[i], new[i]) == 0) {
+			g_print ("Replace %s with %s\n",
+				 old[i], new[i]);
+			return g_strdup (new[i]);
+		}
+		i++;
+	}
+
+	return g_strdup (key);
+}
+
 guchar *
 gnome_print_config_get (GnomePrintConfig *config, const guchar *key)
 {
-	guchar *val;
+	guchar *val, *real_key;
 
 	g_return_val_if_fail (config != NULL, NULL);
 	g_return_val_if_fail (key != NULL, NULL);
 	g_return_val_if_fail (*key != '\0', NULL);
 
-	val = gpa_node_get_path_value (config->node, key);
+	real_key = gnome_print_config_keys_compat (key);
+
+	val = gpa_node_get_path_value (config->node, real_key);
+
+	g_free (real_key);
 
 	return val;
 }
@@ -198,7 +239,6 @@ gnome_print_config_get_boolean (GnomePrintConfig *config, const guchar *key, gbo
 
 	if (v != NULL) {
 		if (!g_ascii_strcasecmp (v, "true") ||
-		    !g_ascii_strcasecmp (v, "yes") ||
 		    !g_ascii_strcasecmp (v, "y") ||
 		    !g_ascii_strcasecmp (v, "yes") ||
 		    (atoi (v) != 0)) {
@@ -444,3 +484,87 @@ gnome_print_config_get_page_size (GnomePrintConfig *config, gdouble *width, gdou
 
 	return TRUE;
 }
+
+
+gboolean
+gnome_print_config_insert_boolean (GnomePrintConfig *config, const guchar *key, gboolean def)
+{
+	GPANode *app;
+
+	g_return_val_if_fail (GNOME_IS_PRINT_CONFIG (config), FALSE);
+	g_return_val_if_fail (key != NULL, FALSE);
+
+	if (strncmp (key, "Settings.Application.", strlen ("Settings.Application."))) {
+		g_warning ("Applications can only append nodes inside the \"Settings.Application\"\n"
+			   "subtree. Node \"%s\" not could not be apppended.", key);
+		return FALSE;
+	}
+
+	app = gpa_node_lookup (config->node, "Settings.Application");
+	if (!app) {
+		g_warning ("Could not find Settings.Application");
+		return FALSE;
+	}
+
+	key += strlen ("Settings.Application.");
+	gpa_key_insert (app, key, def ? "true" : "false");
+
+	return TRUE;
+}
+
+gboolean
+gnome_print_config_insert_options (GnomePrintConfig *config, const guchar *key,
+				   GnomePrintConfigOption *options, const gchar *def)
+{
+	GPANode *app;
+
+	g_return_val_if_fail (GNOME_IS_PRINT_CONFIG (config), FALSE);
+	g_return_val_if_fail (options != NULL, FALSE);
+	g_return_val_if_fail (key != NULL, FALSE);
+
+	if (strncmp (key, "Settings.Application.", strlen ("Settings.Application."))) {
+		g_warning ("Applications can only append nodes inside the \"Settings.Application\"\n"
+			   "subtree. Node \"%s\" not could not be apppended.", key);
+		return FALSE;
+	}
+
+	app = gpa_node_lookup (config->node, "Settings.Application");
+	if (!app) {
+		g_warning ("Could not find Settings.Application");
+		return FALSE;
+	}
+
+	key += strlen ("Settings.Application.");
+	gpa_key_insert (app, key, def);
+
+	return TRUE;
+}
+
+gboolean
+gnome_print_config_get_option (GnomePrintConfig *config, const guchar *key,
+			       GnomePrintConfigOption *options, gint *index)
+{
+	GnomePrintConfigOption option;
+	gchar *value;
+	gint i = 0;
+
+	g_return_val_if_fail (GNOME_IS_PRINT_CONFIG (config), FALSE);
+	g_return_val_if_fail (options != NULL, FALSE);
+	g_return_val_if_fail (key != NULL, FALSE);
+	g_return_val_if_fail (index != NULL, FALSE);
+	
+	*index = 0;
+
+	value = gnome_print_config_get (config, key);
+
+	option = options[i++];
+	while (option.description) {
+		if (strcmp (option.id, value) == 0) {
+			*index = option.index;
+			break;
+		}
+		option = options[i++];
+	}
+
+	return TRUE;
+}
