@@ -167,11 +167,35 @@ gboolean gnome_print_config_set_int     (GnomePrintConfig *config, const guchar
 gboolean gnome_print_config_set_double  (GnomePrintConfig *config, const guchar *key, gdouble val);
 gboolean gnome_print_config_set_length  (GnomePrintConfig *config, const guchar *key, gdouble val, const GnomePrintUnit *unit);
 
+/*
+ * Convenience
+ */
+gboolean  gnome_print_config_get_page_size (GnomePrintConfig *config, gdouble *width, gdouble *height);
+
 /*
  * Debugging
  */
 void gnome_print_config_dump (GnomePrintConfig *gpc);
 
+/*
+ * Add nodes to the configuration object, experimental
+ */
+#ifdef GNOME_PRINT_UNSTABLE_API
+typedef struct _GnomePrintConfigOption GnomePrintConfigOption;
+struct _GnomePrintConfigOption {
+	const gchar *id;
+	const gchar *description;
+	gint index;
+};
+
+gboolean gnome_print_config_insert_boolean (GnomePrintConfig *config, const guchar *key,
+					    gboolean def);
+gboolean gnome_print_config_insert_options (GnomePrintConfig *config, const guchar *key,
+					    GnomePrintConfigOption *options, const gchar *def);
+
+gboolean gnome_print_config_get_option (GnomePrintConfig *config, const guchar *key,
+					GnomePrintConfigOption *options, gint *);
+#endif
 
 /*
  * Paper size
@@ -188,7 +212,6 @@ void gnome_print_config_dump (GnomePrintConfig *gpc);
 #define GNOME_PRINT_KEY_PAPER_SIZE               "Settings.Output.Media.PhysicalSize" /* Paper name, such as A4 or Letter */
 #define GNOME_PRINT_KEY_PAPER_WIDTH              "Settings.Output.Media.PhysicalSize.Width" /* Arbitrary units - use conversion */
 #define GNOME_PRINT_KEY_PAPER_HEIGHT             "Settings.Output.Media.PhysicalSize.Height" /* Arbitrary units - use conversion */
-#define GNOME_PRINT_KEY_PAPER_HEIGHT             "Settings.Output.Media.PhysicalSize.Height" /* Arbitrary units - use conversion */
 #define GNOME_PRINT_KEY_PAPER_ORIENTATION        "Settings.Output.Media.PhysicalOrientation" /* R0, R90, R180, R270 */
 #define GNOME_PRINT_KEY_PAPER_ORIENTATION_MATRIX "Settings.Output.Media.PhysicalOrientation.Paper2PrinterTransform" /* 3x2 abstract matrix */
 
@@ -228,9 +251,10 @@ void gnome_print_config_dump (GnomePrintConfig *gpc);
 #define GNOME_PRINT_KEY_PAPER_MARGIN_BOTTOM "Settings.Output.Media.Margins.Bottom" /* Length, i.e. use conversion */
 
 /* More handy keys */
-#define GNOME_PRINT_KEY_OUTPUT_FILENAME "Settings.Transport.Backend.FileName" /* Filename used when printing to file. */
-#define GNOME_PRINT_KEY_DOCUMENT_NAME "Settings.Document.Name"                /* The name of the document 'Cash flow 2002', `Grandma cookie recipies' */
-#define GNOME_PRINT_KEY_PREFERED_UNIT "Settings.Document.PreferedUnit"        /* Abbreviation for the preferred unit cm, in,... */
+#define GNOME_PRINT_KEY_OUTPUT_FILENAME "Settings.Output.Job.FileName"          /* Filename used when printing to file. */
+#define GNOME_PRINT_KEY_DOCUMENT_NAME   "Settings.Document.Name"                /* The name of the document 'Cash flow 2002', `Grandma cookie recipies' */
+#define GNOME_PRINT_KEY_PREFERED_UNIT   "Settings.Document.PreferedUnit"        /* Abbreviation for the preferred unit cm, in,... */
+
 
 G_END_DECLS
 
