@@ -23,6 +23,8 @@
  *  Copyright (C) 2000-2003 Ximian Inc.
  */
 
+#define GNOME_PRINT_UNSTABLE_API
+
 #include <config.h>
 #include <math.h>
 #include <string.h>
@@ -34,6 +36,7 @@
 #include <libgnomeprint/gpa/gpa-node.h>
 #include <libgnomeprint/gnome-print-private.h>
 #include <libgnomeprint/gnome-print-meta.h>
+#include <libgnomeprint/gnome-print-transport.h>
 #include <libgnomeprint/gnome-print-multipage.h>
 #include <libgnomeprint/gnome-print-job-private.h>
 #include <libgnomeprint/gnome-print-config-private.h>
@@ -110,7 +113,9 @@ gnome_print_job_init (GnomePrintJob *job)
 {
 	job->config = NULL;
 
-	job->meta = gnome_print_meta_new ();
+	job->meta = NULL;
+
+	job->input_file = NULL;
 
 	job->priv = g_new0 (JOBPrivate, 1);
 	job_clear_config_data (job);
@@ -125,12 +130,17 @@ gnome_print_job_finalize (GObject *object)
 
 	if (job->config) {
 		g_object_unref (G_OBJECT (job->config));
+		job->config = NULL;
 	}
 
 	if (job->meta != NULL) {
 		g_object_unref (G_OBJECT (job->meta));
+		job->meta = NULL;
 	}
 
+	g_free (job->input_file);
+	job->input_file = NULL;
+
 	if (job->priv) {
 		job_clear_config_data (job);
 		g_free (job->priv);
@@ -198,13 +208,39 @@ gnome_print_job_get_context (GnomePrintJob *job)
 {
 	g_return_val_if_fail (job != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), NULL);
+	g_return_val_if_fail (job->input_file == NULL, NULL);
 
-	if (job->meta)
-		g_object_ref (G_OBJECT (job->meta));
+	if (!job->meta)
+		job->meta = gnome_print_meta_new ();
+
+	g_object_ref (G_OBJECT (job->meta));
 
 	return job->meta;
 }
 
+/**
+ * gnome_print_job_set_file:
+ * @job: An initialised GnomePrintJob.
+ * @input_file: The file to input from.
+ * 
+ * Set the file to print from. This allows applications to use
+ * gnome-print without using the drawing API by providing the
+ * postscipt output.
+ * 
+ * Return value: None
+ **/
+void
+gnome_print_job_set_file (GnomePrintJob *job, gchar *input_file)
+{
+	g_return_if_fail (job != NULL);
+	g_return_if_fail (GNOME_IS_PRINT_JOB (job));
+	g_return_if_fail (input_file != NULL);
+	g_return_if_fail (job->input_file == NULL);
+	g_return_if_fail (job->meta == NULL);
+
+	job->input_file = g_strdup (input_file);
+}
+
 /**
  * gnome_print_job_get_pages:
  * @job: An initialised and closed GnomePrintJob.
@@ -224,9 +260,8 @@ gnome_print_job_get_pages (GnomePrintJob *job)
 
 	g_return_val_if_fail (job != NULL, 0);
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), 0);
-
-	if (!GNOME_PRINT_JOB_CLOSED (job))
-		return 0;
+	g_return_val_if_fail (job->meta, 0);
+	g_return_val_if_fail (GNOME_PRINT_JOB_CLOSED (job), 0);
 
 	job_update_layout_data (job);
 
@@ -313,6 +348,10 @@ gnome_print_job_close (GnomePrintJob *job)
 
 	g_return_val_if_fail (job != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (job->input_file == NULL, 0);
+
+	if (!job->meta)
+		job->meta = gnome_print_meta_new ();
 
 	pp = job->priv;
 
@@ -326,6 +365,69 @@ gnome_print_job_close (GnomePrintJob *job)
 	return GNOME_PRINT_ERROR_UNKNOWN;
 }
 
+/**
+ * gnome_print_job_metadata_printer:
+ * @job: 
+ * @retval: 
+ * 
+ * We have to special case the Metadata printer where we want to create a file on
+ * disk with the drawing commands to later play them. libgnomeprint needs to be
+ * configured with --with-metadata-printer for it to show up in the printers list
+ * this is used to create streams of drawing commands for example for the regression
+ * test suite.
+ * 
+ * Return Value: TRUE if this job is using the Metadata (to file) printer
+ **/
+static gboolean
+gnome_print_job_metadata_printer (GnomePrintJob *job, gint *retval)
+{
+	GnomePrintTransport *transport;
+	gchar *drivername;
+	gboolean print_to_file = FALSE;
+	const guchar *buf;
+	gint blen;
+
+	drivername = gnome_print_config_get (job->config, "Settings.Engine.Backend.Driver");
+
+	if (drivername == NULL)
+		return FALSE;
+
+	if (strcmp (drivername, "gnome-print-meta") != 0) {
+		g_free (drivername);
+		return FALSE;
+	}
+
+	*retval = GNOME_PRINT_ERROR_UNKNOWN;
+	
+	gnome_print_config_get_boolean (job->config, "Settings.Output.Job.PrintToFile", &print_to_file);
+	if (!print_to_file) {
+		g_warning ("Metadata printer should always be print to file");
+		goto metadata_printer_done;
+	}
+
+	transport = gnome_print_transport_new (job->config);
+	if (!transport) {
+		g_warning ("Could not create transport for metadata printer");
+		goto metadata_printer_done;
+	}
+
+	buf  = gnome_print_meta_get_buffer (GNOME_PRINT_META (job->meta));
+	blen = gnome_print_meta_get_length (GNOME_PRINT_META (job->meta));
+
+	gnome_print_transport_open  (transport);
+	gnome_print_transport_write (transport, buf, blen);
+	gnome_print_transport_write (transport, "GNOME_METAFILE_END", strlen ("GNOME_METAFILE_END"));
+	gnome_print_transport_close (transport);
+	
+	*retval = GNOME_PRINT_OK;
+
+metadata_printer_done:
+
+	g_free (drivername);
+
+	return TRUE;
+}
+
 /**
  * gnome_print_job_print:
  * @job: A closed GnomePrintJob.
@@ -351,6 +453,11 @@ gnome_print_job_print (GnomePrintJob *job)
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (job->priv, GNOME_PRINT_ERROR_UNKNOWN);
 
+	if (job->input_file) {
+		GnomePrintTransport *transport = gnome_print_transport_new (job->config);
+		return gnome_print_transport_print_file (transport, job->input_file);
+	}
+
 	pp = job->priv;
 
 	if (!pp->closed) {
@@ -358,9 +465,6 @@ gnome_print_job_print (GnomePrintJob *job)
 			   "gnome_print_job_print\n");
 		gnome_print_job_close (job);
 	}
-	
-	ctx = gnome_print_context_new (job->config);
-	g_return_val_if_fail (ctx != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 
 	/* Get number of pages in metafile */
 	lpages = gnome_print_meta_get_pages (GNOME_PRINT_META (job->meta));
@@ -368,7 +472,15 @@ gnome_print_job_print (GnomePrintJob *job)
 		return GNOME_PRINT_OK;
 	npages = lpages;
 
-	/* Update layout data */
+	/* Special case where printer is a Metadata print to file */
+	if (gnome_print_job_metadata_printer (job, &ret))
+		return ret;
+
+	/* Create the "real" context */
+	ctx = gnome_print_context_new (job->config);
+	g_return_val_if_fail (ctx != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+
+	/* Update the layout data */
 	is_multipage = FALSE;
 	job_update_layout_data (job);
 	if (pp->LY_LIST) {
@@ -448,6 +560,7 @@ gnome_print_job_render (GnomePrintJob *job, GnomePrintContext *ctx)
 	g_return_val_if_fail (ctx != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (ctx), GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (job->priv, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (job->meta, GNOME_PRINT_ERROR_UNKNOWN);
 
 	pp = job->priv;
 
@@ -494,6 +607,7 @@ gnome_print_job_render_page (GnomePrintJob *job, GnomePrintContext *ctx, gint pa
 	g_return_val_if_fail (ctx != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (ctx), GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (job->priv, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (job->meta, GNOME_PRINT_ERROR_UNKNOWN);
 
 	pp = job->priv;
 
