@@ -23,37 +23,32 @@
  *    Chema Celorio <chema@celorio.com>
  *
  *  Copyright (C) 1999-2001 Ximian Inc. and authors
- *
  */
 
-#define __GNOME_PRINT_C__
-
-#include "config.h"
+#include <config.h>
 #include <string.h>
 #include <gmodule.h>
 
-#ifdef GNOME_PRINT_ENABLE_PAGE_CALLBACKS
-#include <bonobo/bonobo-types.h>
-#endif
+#include <libgnomeprint/gnome-print-i18n.h>
+#include <libgnomeprint/gnome-print-private.h>
+#include <libgnomeprint/gp-gc-private.h>
+#include <libgnomeprint/gnome-print-transport.h>
+#include <libgnomeprint/gnome-print-ps2.h>
+#include <libgnomeprint/gnome-print-pdf.h>
+#include <libgnomeprint/gnome-print-frgba.h>
 
-#include "gnome-print-i18n.h"
-#include "gnome-print-private.h"
-#include "gp-gc-private.h"
-#include "gnome-print-transport.h"
-#include "libgnomeprint-marshal.h"
-
-#include "gnome-print-ps2.h"
-#include "gnome-print-pdf.h"
-#include "gnome-print-frgba.h"
+/* For the buffer stuff, remove when the buffer stuff is moved out here */
+#include <sys/mman.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <unistd.h>
+/* Endif */
 
 static void gnome_print_context_class_init (GnomePrintContextClass *klass);
 static void gnome_print_context_init (GnomePrintContext *pc);
 static void gnome_print_context_finalize (GObject *object);
 
-#ifdef GNOME_PRINT_ENABLE_MODULE_DRIVERS
-static GnomePrintContext *gnome_print_context_create (gpointer get_type, GnomePrintConfig *config);
-#endif
-
 static GObjectClass *parent_class = NULL;
 
 GType
@@ -148,7 +143,11 @@ gnome_print_context_create_transport (GnomePrintContext *pc)
 	g_return_val_if_fail (pc->transport == NULL, GNOME_PRINT_ERROR_UNKNOWN);
 
 	pc->transport = gnome_print_transport_new (pc->config);
-	g_return_val_if_fail (pc->transport != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	
+	if (pc->transport == NULL) {
+		g_warning ("Could not create transport inside gnome_print_context_create_transport");
+		return GNOME_PRINT_ERROR_UNKNOWN;
+	}
 
 	return GNOME_PRINT_OK;
 }
@@ -223,15 +222,7 @@ gnome_print_showpage (GnomePrintContext *pc)
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-
-#ifdef ALLOW_BROKEN_PGL
-	if (!pc->haspage) {
-		g_warning ("file %s: line %d: Missing beginpage in print job", __FILE__, __LINE__);
-		gnome_print_beginpage (pc, "Unnamed");
-	}
-#else
 	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
-#endif
 
 	ret = GNOME_PRINT_OK;
 
@@ -265,15 +256,7 @@ gnome_print_gsave (GnomePrintContext *pc)
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-
-#ifdef ALLOW_BROKEN_PGL
-	if (!pc->haspage) {
-		g_warning ("file %s: line %d: Missing beginpage in print job", __FILE__, __LINE__);
-		gnome_print_beginpage (pc, "Unnamed");
-	}
-#else
 	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
-#endif
 
 	ret = GNOME_PRINT_OK;
 
@@ -303,15 +286,7 @@ gnome_print_grestore (GnomePrintContext *pc)
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-
-#ifdef ALLOW_BROKEN_PGL
-	if (!pc->haspage) {
-		g_warning ("file %s: line %d: Missing beginpage in print job", __FILE__, __LINE__);
-		gnome_print_beginpage (pc, "Unnamed");
-	}
-#else
 	g_return_val_if_fail (pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
-#endif
 
 	ret = GNOME_PRINT_OK;
 
@@ -405,81 +380,6 @@ gnome_print_glyphlist_transform (GnomePrintContext *pc, const gdouble *affine, G
 	return GNOME_PRINT_OK;
 }
 
-
-#ifdef GNOME_PRINT_ENABLE_PAGE_CALLBACKS
-gint
-gnome_print_callback_closure_invoke (GnomePrintContext *pc,
-				     gpointer           pagedata,
-				     gpointer           docdata,
-				     GClosure          *closure)
-{
-	guint  ret;
-
-	bonobo_closure_invoke (
-		closure, G_TYPE_UINT, &ret,
-		GNOME_TYPE_PRINT_CONTEXT, pc,
-		G_TYPE_POINTER, pagedata,
-		G_TYPE_POINTER, docdata, 0);
-
-	return ret;
-}
-
-gint
-gnome_print_page_callback_closure (GnomePrintContext *pc,
-				   const guchar      *name,
-				   gpointer           pagedata,
-				   gpointer           docdata,
-				   GClosure          *closure)
-{
-	guint  ret;
-	GValue ret_val = {0, };
-
-	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (pc->gc, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (!pc->haspage, GNOME_PRINT_ERROR_NOPAGE);
-	g_return_val_if_fail (closure != NULL, GNOME_PRINT_ERROR_BADVALUE);
-
-	g_value_init (&ret_val, G_TYPE_INT);
-
-	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->page)
-		return GNOME_PRINT_CONTEXT_GET_CLASS (pc)->page (
-			pc, name, pagedata, docdata, closure);
-
-	ret = gnome_print_beginpage (pc, name);
-	g_return_val_if_fail (ret == GNOME_PRINT_OK, ret);
-
-	ret = gnome_print_callback_closure_invoke (
-		pc, pagedata, docdata, closure);
-	g_return_val_if_fail (ret == GNOME_PRINT_OK, ret);
-
-	ret = gnome_print_showpage (pc);
-	g_return_val_if_fail (ret == GNOME_PRINT_OK, ret);
-
-	return ret;
-}
-
-
-gint
-gnome_print_page_callback (GnomePrintContext *pc, GnomePrintPageCallback callback,
-			   const guchar *name, gpointer pagedata, gpointer docdata)
-{
-	GClosure *closure;
-	gint      ret;
-
-	closure = bonobo_closure_store (
-		g_cclosure_new (G_CALLBACK (callback), NULL, NULL),
-		libgnomeprint_marshal_INT__OBJECT_POINTER_POINTER);
-
-	ret = gnome_print_page_callback_closure (
-		pc, name, pagedata, docdata, closure);
-
-	g_closure_unref (closure);
-
-	return ret;
-}
-#endif /* GNOME_PRINT_ENABLE_PAGE_CALLBACKS */
-
 /**
  * gnome_print_context_close:
  * @pc: A #GnomePrintContext
@@ -506,11 +406,14 @@ gnome_print_context_close (GnomePrintContext *pc)
 	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->close)
 		ret = GNOME_PRINT_CONTEXT_GET_CLASS (pc)->close (pc);
 
-	g_return_val_if_fail (ret == GNOME_PRINT_OK, GNOME_PRINT_ERROR_UNKNOWN);
+	if (ret != GNOME_PRINT_OK) {
+		g_warning ("Could not close transport inside gnome_print_context_close");
+		return ret;
+	}
 
 	if (pc->transport) {
-		g_warning ("file %s: line %d: Closing Context did not clear transport", __FILE__, __LINE__);
-		return GNOME_PRINT_ERROR_UNKNOWN;
+		g_warning ("file %s: line %d: Closing Context should clear transport", __FILE__, __LINE__);
+		return ret;
 	}
 
 	return ret;
@@ -520,22 +423,23 @@ gnome_print_context_close (GnomePrintContext *pc)
  * gnome_print_context_new:
  * @config: GnomePrintConfig object to query print settings from
  *
- * This method gives you new #GnomePrintContext object, associated with given
- * #GnomePrinter. You should use the resulting object as 'black box', without
- * assuming anything about it's type, as depending on situation appropriate
- * wrapper context may be used instead of direct driver.
+ * Create new printing context from config. You have to have set
+ * all the options/settings beforehand, as changing the config of
+ * an existing context has undefined results.
  *
- * Returns: The new #GnomePrintContext or %NULL, if there is an error
+ * Also, if creating the context by hand, it completely ignores layout and
+ * orientation value. If you need those, use GnomePrintJob. The
+ * later also can create output context for you, so in most cases
+ * you may want to ignore gnome_print_context_new at all.
+ *
+ * Returns: The new GnomePrintContext or NULL on error
  */
-
 GnomePrintContext *
 gnome_print_context_new (GnomePrintConfig *config)
 {
 	GnomePrintContext *pc = NULL;
 	guchar *drivername;
-#ifdef GNOME_PRINT_ENABLE_MODULE_DRIVERS
-	guchar *modulename;
-#endif
+
 	g_return_val_if_fail (config != NULL, NULL);
 
 	drivername = gnome_print_config_get (config, "Settings.Engine.Backend.Driver");
@@ -545,7 +449,6 @@ gnome_print_context_new (GnomePrintConfig *config)
 
 	if (strcmp (drivername, "gnome-print-ps") == 0) {
 		GnomePrintContext *ps;
-		g_print ("Printing to Postscript...\n");
 		ps = gnome_print_ps2_new (config);
 		if (ps == NULL)
 			return NULL;
@@ -558,7 +461,6 @@ gnome_print_context_new (GnomePrintConfig *config)
 	}
 	
 	if (strcmp (drivername, "gnome-print-pdf") == 0) {
-		g_print ("Printing to PDF ...\n");
 		pc = gnome_print_pdf_new (config);
 		if (pc == NULL)
 			return NULL;
@@ -566,61 +468,52 @@ gnome_print_context_new (GnomePrintConfig *config)
 		return pc;
 	}
 
-#ifdef GNOME_PRINT_ENABLE_MODULE_DRIVERS
-        /* FIXME: This has not been tested yet (Chema) */
-	modulename = gnome_print_config_get (config, "Settings.Engine.Backend.Driver.Module");
-	if (modulename) {
-		static GHashTable *modules = NULL;
-		GModule *module;
-		if (!modules)
-			modules = g_hash_table_new (g_str_hash, g_str_equal);
-		module = g_hash_table_lookup (modules, modulename);
-		if (!module) {
-			gchar *path;
-			path = g_module_build_path (GNOME_PRINT_MODULES_DIR "/drivers", modulename);
-			module = g_module_open (path, G_MODULE_BIND_LAZY);
-			if (module)
-				g_hash_table_insert (modules, g_strdup (modulename), module);
-			g_free (path);
-		}
-		if (module) {
-			gpointer get_type;
-			if (g_module_symbol (module, "gnome_print__driver_get_type", &get_type)) {
-				pc = gnome_print_context_create (get_type, config);
-			} else {
-				g_warning ("Missing gnome_print__driver_get_type in %s\n", modulename);
-				g_module_close (module);
-			}
-		} else {
-			g_warning ("Cannot open module: %s\n", modulename);
-		}
-		g_free (modulename);
-	} else {
-		g_warning ("Unknown driver: %s", drivername);
-	}
+	if (pc == NULL)
+		g_warning ("Could not create context for driver: %s", drivername);
 
-#endif
 	g_free (drivername);
 
 	return pc;
 }
 
-#ifdef GNOME_PRINT_ENABLE_MODULE_DRIVERS
-static GnomePrintContext *
-gnome_print_context_create (gpointer get_type, GnomePrintConfig *config)
+
+void
+gnome_print_buffer_munmap (GnomePrintBuffer *b)
 {
-	GnomePrintContext *pc;
-	GType (* driver_get_type) (void);
-	GType type;
+	if (b->buf)
+		munmap (b->buf, b->buf_size);
+	b->buf = NULL;
+	b->buf_size = 0;
+}
 
-	driver_get_type = get_type;
+gint
+gnome_print_buffer_mmap (GnomePrintBuffer *b,
+			 const guchar *file_name)
+{
+	struct stat s;
+	gint fh;
 
-	type = (* driver_get_type) ();
-	g_return_val_if_fail (g_type_is_a (type, GNOME_TYPE_PRINT_CONTEXT), NULL);
+	b->buf = NULL;
+	b->buf_size = 0;
 
-	pc = g_object_new (type, NULL);
-	gnome_print_context_construct (pc, config);
+	fh = open (file_name, O_RDONLY);
+	if (fh < 0) {
+		g_warning ("Can't open \"%s\"", file_name);
+		return GNOME_PRINT_ERROR_UNKNOWN;
+	}
+	if (fstat (fh, &s) != 0) {
+		g_warning ("Can't stat \"%s\"", file_name);
+		return GNOME_PRINT_ERROR_UNKNOWN;
+	}
+	
+	b->buf = mmap (NULL, s.st_size, PROT_READ, MAP_SHARED, fh, 0);
+	b->buf_size = s.st_size;
+	
+	close (fh);
+	if ((b->buf == NULL) || (b->buf == (void *) -1)) {
+		g_warning ("Can't mmap file %s", file_name);
+		return GNOME_PRINT_ERROR_UNKNOWN;
+	}
 
-	return pc;
+	return GNOME_PRINT_OK;
 }
-#endif
