@@ -183,7 +183,6 @@ gpa_config_new (void)
 	GPAConfig *config = NULL;
 	GPANode *printer;
 	GPANode *settings = NULL;
-	GPANode *def;
 
 	gpa_init ();
 	
@@ -193,18 +192,7 @@ gpa_config_new (void)
 		goto gpa_config_new_error;
 	}
 
-	def = gpa_printer_get_default_settings (GPA_PRINTER (printer));
-	if (def == NULL) {
-		g_warning ("Could not get default settings for %s\n",
-			   gpa_node_id (printer));
-		goto gpa_config_new_error;
-	}
-
-	settings = gpa_node_duplicate (def);
-	if (!settings) {
-		g_warning ("Could not duplicate default settings");
-		goto gpa_config_new_error;
-	}
+	settings = gpa_printer_get_default_settings (GPA_PRINTER (printer));
 
 	config = gpa_config_new_full (GPA_PRINTER (printer), GPA_SETTINGS (settings));
 
@@ -345,7 +333,7 @@ gpa_config_from_string (const gchar *str, guint flags)
 		break;
 	}
 	if (!node) {
-		g_print ("not node\n");
+		g_warning ("Could not find the selected settings in the settings list");
 		goto config_from_string_done;
 	}
 
@@ -378,7 +366,7 @@ gpa_config_from_string (const gchar *str, guint flags)
 	
 	new_settings = (GPASettings *) gpa_settings_new_from_model_and_tree (GPA_NODE (model), node);
 	if (!new_settings) {
-		g_print ("Could not create settings\n");
+		g_warning ("Could not create settings from model and tree\n");
 		goto config_from_string_done;
 	}
 
