@@ -118,7 +118,8 @@ gpa_key_finalize (GObject *object)
 	}
 	node->children = NULL;
 
-	gpa_node_unref (key->option);
+	if (key->option)
+		gpa_node_unref (key->option);
 	if (key->value)
 		g_free (key->value);
 
@@ -138,6 +139,10 @@ gpa_key_duplicate (GPANode *node)
 
 	new = (GPAKey *) gpa_node_new (GPA_TYPE_KEY, gpa_node_id (node));
 
+	if ((GPA_NODE_FLAGS (node) & NODE_FLAG_LOCKED) == NODE_FLAG_LOCKED) {
+		GPA_NODE_SET_FLAGS (key, NODE_FLAG_LOCKED);
+	}
+	
 	if (key->value)
 		new->value = g_strdup (key->value);
 	new->option = gpa_node_ref (key->option);
@@ -191,6 +196,14 @@ gpa_key_set_value (GPANode *node, const guchar *value)
 	key    = GPA_KEY (node);
 	option = GPA_KEY_OPTION (key);
 
+	if (option == NULL) {
+		   g_free (key->value);
+		   key->value = g_strdup (value);
+		   return TRUE;
+	}
+	
+	g_return_val_if_fail (option != NULL, FALSE);
+
 	if (strcmp (key->value, value) == 0)
 		return FALSE;
 
@@ -207,13 +220,13 @@ gpa_key_set_value (GPANode *node, const guchar *value)
 		gpa_node_unref (child);
 		break;
 	case GPA_OPTION_TYPE_STRING:
+	case GPA_OPTION_TYPE_KEY:
 		g_free (key->value);
 		key->value = g_strdup (value);
 		break;
 	default:
-		if (FALSE)
-			g_warning ("Cant set value of %s to %s, set value for type not set. Current val:%s",
-				   gpa_node_id (node), value, key->value);
+		g_warning ("Cant set value of %s to %s, set value for type not set. Current val:%s (%d)",
+			   gpa_node_id (node), value, key->value, option->type);
 		return FALSE;
 	}
 
@@ -352,7 +365,6 @@ gpa_key_merge_from_key (GPAKey *dst, GPAKey *src)
 	g_return_val_if_fail (GPA_IS_KEY (dst), FALSE);
 	g_return_val_if_fail (src != NULL, FALSE);
 	g_return_val_if_fail (GPA_IS_KEY (src), FALSE);
-	g_return_val_if_fail (src->option != NULL, FALSE);
 
 	if (dst->value)
 		g_free (dst->value);
@@ -360,7 +372,10 @@ gpa_key_merge_from_key (GPAKey *dst, GPAKey *src)
 
 	if (dst->option)
 		gpa_node_unref (dst->option);
-	dst->option = gpa_node_ref (src->option);
+	if (src->option)
+		dst->option = gpa_node_ref (src->option);
+	else
+		dst->option = NULL;
 
 	gpa_key_merge_children_from_key (dst, src);
 
@@ -385,7 +400,7 @@ gpa_key_merge_children_from_option (GPAKey *key, GPAOption *option)
 		options = g_slist_prepend (options, child);
 		child = child->next;
 	}
-#if 0
+#ifdef DEBUG
 {
 	GSList *l;
 	l = keys;
@@ -410,7 +425,7 @@ gpa_key_merge_children_from_option (GPAKey *key, GPAOption *option)
 		while (l) {
 			if (GPA_NODE (options->data)->qid == GPA_NODE (l->data)->qid) {
 				GPAKey *key = l->data;
-#if 0		
+#ifdef DEBUG	
 				g_print ("IS IN BOTH %s\n", gpa_node_id (l->data));
 #endif
 				if (key->value == NULL)
@@ -429,7 +444,7 @@ gpa_key_merge_children_from_option (GPAKey *key, GPAOption *option)
 			l = l->next;
 		}
 		if (l == NULL) {
-#if 0
+#ifdef DEBUG
 			g_print ("IS ONLY IN option, add: %s\n", gpa_node_id (options->data));
 #endif
 			/* Is only in new */
@@ -441,7 +456,7 @@ gpa_key_merge_children_from_option (GPAKey *key, GPAOption *option)
 	}
 	g_slist_free (keys2);
 	while (keys) {
-#if 0		
+#ifdef DEBUG
 		g_print ("IS ONLY IN keys, remove %s\n", gpa_node_id (keys->data));
 #endif
 		/* Is only in old */
@@ -504,3 +519,28 @@ gpa_key_merge_from_option (GPAKey *key, GPAOption *option)
 
 	return TRUE;
 }
+
+
+gboolean
+gpa_key_insert (GPANode *parent, const guchar *path, const guchar *value)
+{
+	GPANode *new;
+	const guchar *dot;
+	
+	g_return_val_if_fail (GPA_IS_KEY (parent), FALSE);
+	g_return_val_if_fail (path != NULL, FALSE);
+
+	/* For now, only insert first level keys. (Chema) */
+	dot = strchr (path, '.');
+	if (dot) {
+		g_print ("We only support top level key_inserts");
+		return FALSE;
+	}
+
+	new = gpa_node_new (GPA_TYPE_KEY, path);
+	if (value)
+		GPA_KEY (new)->value = g_strdup (value);
+	gpa_node_attach (parent, new);
+
+	return TRUE;
+}
