@@ -136,6 +136,11 @@ gpa_option_duplicate (GPANode *node)
 	option = GPA_OPTION (node);
 
 	new_node = gpa_node_new (GPA_TYPE_OPTION, gpa_node_id (node));
+
+	if ((GPA_NODE_FLAGS (node) & NODE_FLAG_LOCKED) == NODE_FLAG_LOCKED) {
+		GPA_NODE_SET_FLAGS (new_node, NODE_FLAG_LOCKED);
+	}
+	
 	new = GPA_OPTION (new_node);
 	new->type = option->type;
 
@@ -230,53 +235,34 @@ gpa_option_create_key (GPAOption *option, GPANode *parent)
 	GPANode *child;
 	GPAKey *key;
 
-	key = NULL;
-
-	switch (option->type) {
-	case GPA_OPTION_TYPE_NODE:
-	case GPA_OPTION_TYPE_KEY:
-	case GPA_OPTION_TYPE_STRING:
-		key_node = gpa_node_new (GPA_TYPE_KEY, GPA_NODE_ID (option));
-		key = GPA_KEY (key_node);
-		key->option = gpa_node_ref (GPA_NODE (option));
-		if (option->value)
-			key->value = g_strdup (option->value);
-
-		child = GPA_NODE (option)->children;
-		while (child) {
-			GPANode *child_key;
-			child_key = gpa_option_create_key (GPA_OPTION (child),
-							   key_node);
-			if (child_key) {
-				gpa_node_attach (key_node, child_key);
-			}
-			child = child->next;
-		}
-		gpa_node_reverse_children (GPA_NODE (key));
-		break;
-	case GPA_OPTION_TYPE_LIST:
-		key_node = gpa_node_new (GPA_TYPE_KEY, GPA_NODE_ID (option));
-		key = GPA_KEY (key_node);
-		key->option = gpa_node_ref (GPA_NODE (option));
+	key_node = gpa_node_new (GPA_TYPE_KEY, GPA_NODE_ID (option));
+	key = (GPAKey *) key_node;
+	key->option = gpa_node_ref (GPA_NODE (option));
+	if (option->value)
 		key->value = g_strdup (option->value);
-		child = gpa_option_get_child_by_id
-			(option, option->value);
+
+	if (option->type == GPA_OPTION_TYPE_LIST) {
+		child = gpa_option_get_child_by_id (option, option->value);
 		child = child->children;
-		while (child) {
-			GPANode *child_key;
-			child_key = gpa_option_create_key (GPA_OPTION (child),
-							   key_node);
-			if (child_key) {
-				gpa_node_attach (key_node, child_key);
-			}
-			child = child->next;
-		}
-		gpa_node_reverse_children (GPA_NODE (key));
-		break;
-	default:
-		g_assert_not_reached ();
+	} else {
+		child = GPA_NODE (option)->children;
 	}
 
+	if ((GPA_NODE_FLAGS (option) & NODE_FLAG_LOCKED) == NODE_FLAG_LOCKED) {
+		GPA_NODE_SET_FLAGS (key, NODE_FLAG_LOCKED);
+	}
+
+	while (child) {
+		GPANode *child_key;
+		child_key = gpa_option_create_key (GPA_OPTION (child),
+						   key_node);
+		if (child_key) {
+			gpa_node_attach (key_node, child_key);
+		}
+		child = child->next;
+	}
+	gpa_node_reverse_children (GPA_NODE (key));
+
 	return (GPANode *) key;
 }
 
@@ -352,6 +338,7 @@ gpa_option_new_from_tree (xmlNodePtr tree, GPANode *parent)
 	GPANode *option = NULL;
 	xmlChar *id;
 	xmlChar *type = NULL;
+	xmlChar *locked = NULL;
 
 	g_return_val_if_fail (tree != NULL, NULL);
 
@@ -380,20 +367,30 @@ gpa_option_new_from_tree (xmlNodePtr tree, GPANode *parent)
 	
 	if (!type || !type[0]) {
 		option = gpa_option_node_new_from_tree (tree, parent, id);
-		goto new_from_tree_done;
+		goto new_from_tree_option_loaded;
 	}
 
 	if (strcmp (type, "List") == 0) {
 		option = gpa_option_list_new_from_tree (tree, parent, id);
-		goto new_from_tree_done;
+		goto new_from_tree_option_loaded;
 	}
 
 	if (strcmp (type, "String") == 0) {
 		option = gpa_option_string_new_from_tree (tree, parent, id);
-		goto new_from_tree_done;
+		goto new_from_tree_option_loaded;
 	}
 
+new_from_tree_option_loaded:
+	locked = xmlGetProp (tree, "Locked");
+	
+	if (locked && !strcmp (locked, "true")) {
+		GPA_NODE_SET_FLAGS (option, NODE_FLAG_LOCKED);
+	} else {
+		GPA_NODE_UNSET_FLAGS (option, NODE_FLAG_LOCKED);
+	}		
+	
 new_from_tree_done:
+	my_xmlFree (locked);
 	my_xmlFree (id);
 	my_xmlFree (type);
 
